import React, { memo, useEffect, useState, Fragment } from 'react';
import { View, Image, ScrollView, publishDps } from '@ray-js/ray';
import PercentSlider from '@ray-js/lamp-percent-slider';
import { useActions, useProps } from '@ray-js/panel-sdk';
import Popup from '@ray-js/components-ty-popup';
import iconv from 'iconv-lite';

import styles from './index.module.less';
import { IMusicList } from './type';

const tagList = ['TF卡', 'USB'];
const listMode = ['curr_list_tf', 'curr_list_usb', 'curr_list_flash'];

/**
 * 当前包数
 */
let cur_package_num = 1;
/**
 * 总包数
 */
let total_num = 0;
/**
 * 歌曲id
 */
let music_id = 1;

const Device = props => {
  const dpaction = useActions();
  const list_pp = useProps(props => props.list_pp); // 播放状态
  const list_select = useProps(props => props.list_select); // 列表选择 01 01 表示 存在 TF卡 和 USB
  const curr_list = useProps(props => props.curr_list); // 当前列表
  const list_volume = useProps(props => props.list_volume); // 音量
  const search_list_track = useProps(props => props.search_list_track); // 查询的列表歌曲数据
  const list_track = useProps(props => props.list_track); // 当前播放id

  const [tag, setTag] = useState(0);
  const [vol, setVol] = useState(0);
  const [isShowFavorite, setIsShowFavorite] = useState(false);
  const [musicList, setMusicList] = useState<IMusicList[]>([]);
  const [musicName, setMusicName] = useState('暂无歌曲');

  useEffect(() => {
    tagList.forEach(async (_item, index) => {
      await dpaction.search_list_track.set((index + 1).toString(16).padStart(2, '0'));
    });
  }, []);

  useEffect(() => {
    curr_list === listMode[0] ? setTag(0) : curr_list === listMode[1] ? setTag(1) : '';
  }, [curr_list]);

  useEffect(() => {
    setVol(list_volume);
  }, [list_volume]);

  useEffect(() => {
    const num = parseInt(search_list_track.slice(2, 4), 16);
    if (num === 0) {
      // 判断没有音乐列表
      setMusicList(preMusicList => {
        const name = tagList[tag];
        const newMusicList = preMusicList.map(item => {
          if (item.name === name) {
            item.musics = [];
            item.hex = '';
            return item;
          }
          return { ...item };
        });
        return newMusicList;
      });
      return;
    }
    total_num === 0 ? (total_num = parseInt(search_list_track.slice(0, 2), 16)) : '';
    if (num !== cur_package_num) return;
    if (cur_package_num === 1) {
      // 第一包的数据
      const hex = search_list_track.slice(6);
      setMusicList(preMusicList => {
        const name = tagList[tag];
        const newMusicList = [...preMusicList];
        newMusicList.push({ name, musics: [], hex: hex });
        return newMusicList;
      });
    } else {
      // 拼包数据
      setMusicList(preMusicList => {
        const newMusicList = preMusicList.map((item, i) => {
          if (i === preMusicList.length - 1) {
            // 最后一个数组对象添加数据
            const hex = item.hex + search_list_track.slice(6);
            return { ...item, hex };
          }
          return item;
        });

        return newMusicList;
      });
    }
    cur_package_num++;
    if (total_num !== num) return;
    total_num = 0;
    cur_package_num = 1;
    music_id = 1;
  }, [search_list_track]);

  useEffect(() => {
    if (musicList.length === 0) return;
    if (total_num !== 0 || cur_package_num !== 1) return;
    const music = musicList.find(item => item.musics.length === 0);
    if (!music) return;
    // handleParseGB2312();
    console.log(111, musicList, musicList[0].hex);
  }, [musicList]);

  useEffect(() => {
    const music = musicList.find(item => item.name === tagList[tag]);
    if (!music) return;
    const musics: { name: string; id: number }[] = [];
    music.musics.forEach(item => {
      item.data.forEach(music => musics.push(music));
    });
    const cur_music = musics.find(item => item.id === list_track);
    setMusicName(cur_music?.name || '暂无歌曲');
  }, [list_track, musicList]);

  /**
   * 处理点击播放暂停
   */
  const handlePlay = () => {
    dpaction.list_pp.toggle({ throttle: 300 });
  };

  /**
   * 处理点击切换播放列表模式
   */
  const handleChangeTag = async (index: number) => {
    await dpaction.curr_list.set(listMode[index]);
    // await dpaction.search_list_track.set((index + 1).toString(16).padStart(2, '0'));
  };

  /**
   * 处理滑动条滑动时
   * @param value 音量
   */
  const handleChangeSlider = (value: number) => {
    setVol(value);
  };

  /**
   * 出来滑动条抬起
   * @param value 音量
   */
  const handleSliderEnd = (value: number) => {
    dpaction.list_volume.set(value);
  };

  /**
   * 解析gb2312编码
   */
  const handleParseGB2312 = () => {
    const newMusicList = musicList.map((item, index) => {
      if (!item.hex) return item;

      const gb2312Buffer = Buffer.from(item.hex, 'hex');
      console.log(2222, gb2312Buffer);

      const utf8String = iconv.decode(gb2312Buffer, 'gb2312');
      const result = utf8String.split('\r\n');

      result.forEach((name, i) => {
        if (name.startsWith('F:')) {
          item.musics.push({
            file: name.replace('F:', ''),
            data: [],
            open: false,
          });
        } else {
          item.musics[item.musics.length - 1].data.push({
            name: name.replace('.mp3', ''),
            id: music_id++,
          });
        }
      });

      return { ...item };
    });
    setMusicList(newMusicList);
  };

  /**
   * 处理点击文件夹
   */
  const handleOpenFiles = (i: number) => {
    setMusicList(preMusicList => {
      const newMusicList = preMusicList.map(item => {
        if (item.name === tagList[tag]) {
          item.musics[i].open = !item.musics[i].open;
        }
        return { ...item };
      });

      return newMusicList;
    });
  };

  /**
   * 渲染文件夹列表
   */
  const RenderFiles = () => {
    const rootFiles = musicList.find(item => item.name === tagList[tag]);

    if (!rootFiles || !rootFiles.musics) {
      return <View className={styles.no_music}>暂无音乐</View>;
    }
    return (
      <>
        {rootFiles.musics.map((item, index) => (
          <Fragment key={item.file}>
            <View
              className={styles.listLabel}
              onClick={() => {
                handleOpenFiles(index);
              }}
            >
              {item.file}
              {item.open ? (
                <Image src={require('@/static/imgs/common/up_ic.png')} />
              ) : (
                <Image src={require('@/static/imgs/common/down_ic.png')} />
              )}
            </View>
            <>
              <View className={`${styles.list}`}>
                {item.data.map(music => (
                  <Fragment key={music.id}>
                    {item.open && (
                      <View
                        className={`${styles.item} ${music.id === list_track &&
                          styles.currentMusic}`}
                        onClick={() => {}}
                        id={`music_${music.id}`}
                      >
                        {music.name}
                      </View>
                    )}
                  </Fragment>
                ))}
              </View>
            </>
          </Fragment>
        ))}
      </>
    );
  };

  return (
    <View className={styles.deviceContainer}>
      <View className={styles.content}>
        {/* 标签区域 */}
        <View className={styles.tag}>
          {tagList.map((item: string, index: number) => (
            <View
              className={styles.item + ' ' + (index === tag && styles.current)}
              key={index}
              onClick={() => {
                handleChangeTag(index);
              }}
            >
              {item}
            </View>
          ))}
        </View>
        {/* 曲目列表区域 */}
        <View className={styles.scrollView}>
          <ScrollView
            refresherTriggered
            scrollY
            className={styles.files}
            scroll-into-view={`music_${list_track}`}
          >
            {RenderFiles()}
          </ScrollView>
        </View>
      </View>
      {/* 控制组件区域 */}
      <View className={styles.control}>
        <View className={styles.musicName}>{musicName}</View>
        <View className={styles.percentSlider}>
          <Image src={require('@/static/imgs/common/vol_ic.png')}></Image>
          <View style={{ flex: 1 }}>
            <PercentSlider
              min={1}
              max={100}
              showIcon={false}
              showText={false}
              value={vol}
              barStyle={{
                height: 'var(--thumb-width)',
                background: 'linear-gradient(357deg, #FF5252 0%, #FFDD7E 99%)',
              }}
              thumbStyle={{
                // @ts-ignore
                width: 'var(--thumb-width)',
                boxShadow: 'none',
              }}
              trackStyle={{
                height: 'var(--thumb-width)',
                borderRadius: '31.395rpx',
                overflow: 'hidden',
              }}
              onChange={handleChangeSlider}
              onTouchEnd={handleSliderEnd}
            />
          </View>
          <View className={styles.progress}>{vol}%</View>
        </View>
        <View className={styles.playBox}>
          <View className={styles.loop} style={{ marginRight: '80rpx' }}>
            <Image src={require('@/static/imgs/common/favorite_list_ic.png')} />
          </View>
          <View
            className={styles.last}
            onClick={() => {
              dpaction.list_prev_next.set('last', { throttle: 300 });
            }}
          >
            <Image src={require('@/static/imgs/home/last_ic.png')} />
          </View>
          <View className={styles.play} onClick={() => handlePlay()}>
            {!list_pp ? (
              <Image src={require('@/static/imgs/home/play_ic.png')} />
            ) : (
              <Image src={require('@/static/imgs/home/stop_ic.png')} />
            )}
          </View>
          <View
            className={styles.next}
            onClick={() => {
              dpaction.list_prev_next.set('next', { throttle: 300 });
            }}
          >
            <Image src={require('@/static/imgs/home/next_ic.png')} />
          </View>
          <View className={styles.loop} style={{ marginLeft: '80rpx' }}>
            <Image src={require('@/static/imgs/common/order_ic.png')} />
          </View>
        </View>
      </View>

      {/* 收藏列表弹框组件区域 */}
      <Popup
        show={isShowFavorite}
        position="bottom"
        round
        customStyle={{
          width: '100%',
          height: '80%',
          background: '#FFF',
          overflow: 'hidden',
        }}
        safeAreaInsetBottom={true}
      ></Popup>
    </View>
  );
};

export default memo(Device);
